/*
*/
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <ctype.h>
#include <string.h>

#define LINE_SIZE 256

//-------------------------------------------------------------------

int script_rstrip(char *line)
{
  int pos = strlen(line);
  while (pos>0 && (line[pos-1]==' '||line[pos-1]=='\t'||line[pos-1]=='\n'||line[pos-1]=='\r')) line[--pos]='\0';
  return pos;
}

int script_strip(FILE *f, char *program, const int program_size, char *param_buf, int *param_size)
{
  char *pls, *prem;
  char line[LINE_SIZE];
  int ferr=0, fsize=0, fpsize=0, frem=0, fnostrip=0;
  int ple, eqc=0, rc=0, bc=0;
  
  if (program_size>0 && program) {
    program[0]='\0';
    fsize=1;
    if (param_size>0 && param_buf) {
      param_buf[0]='\0';
      fpsize=1;
    }
  } else {
    *param_size=0;
  }
  while (!ferr && !fnostrip && fgets(line,LINE_SIZE,f)) {
    pls=line;
    while (pls[0]==' '||pls[0]=='\t') ++pls;
    if (script_rstrip(pls)>0)
    {
      if (pls[0]=='@') {
        fnostrip=(strncmp(pls, "@nostrip", 8)==0);
        ple = strlen(pls);
        pls[ple+1]='\0';
        pls[ple++]='\n';
        bc+=ple;
        if (fpsize) {
          if (bc<=*param_size) strcat(param_buf, pls); else ferr=1;
        }
        pls[0]='\0';
      }
      if (pls[0] && !frem && pls[0]=='-' && pls[1]=='-' && pls[2]=='[') {
        prem=pls+3;
        eqc=0;
        while (prem[0]=='=') {
          ++prem;
          ++eqc;
        }
        if (prem[0]=='[') {
          frem=1;
          pls=++prem;
        }
      }
      if (pls[0] && frem) {
          prem = pls;
          for (; (prem = strchr(prem, ']'))!=0 && frem; prem++) {
            ple=0;
            ++prem;
            while ((eqc-ple)>0 && prem[ple]=='=') ++ple;
            if (ple==eqc && prem[ple]==']') {
              pls=prem+ple+1;
              while (pls[0]==' '||pls[0]=='\t') ++pls;
              frem=0;
            }
         }
      }
      if (pls[0] && (frem || (pls[0]=='-' && pls[1]=='-') ||
          (pls[0]=='r' && pls[1]=='e' && pls[2]=='m'))) pls[0]='\0';

      if (pls[0]) {
        prem = pls;
        for (; (prem = strchr(prem, '-'))!=0; prem++) 
          if (strncmp(prem, "--", 2)==0 && strchr(prem,'"')==0 && strchr(prem,'\'')==0){
            prem[0]='\0';
            script_rstrip(pls);
          }
      }

      ple = strlen(pls);
      if (ple>0) {
        pls[ple+1]='\0';
        pls[ple++]='\n';
        rc+=ple;
        if (fsize) {
          if (rc<=program_size) strcat(program, pls); else ferr=1;
        }
      }
    }
  }
  if (fnostrip || frem) {
    rc=0;
    bc=0;
    if (fsize) program[0]='\0';
    fseek(f,0, SEEK_SET);
    while (!ferr && fgets(line,LINE_SIZE,f)) {
      pls=line;
      while (pls[0]==' '||pls[0]=='\t') ++pls;
      if (!(strncmp(pls, "@nostrip", 8)==0)) {
        pls=line;
        script_rstrip(pls);
        ple = strlen(pls);
        if (pls[ple-1]!='\n') {
          pls[ple+1]='\0';
          pls[ple++]='\n';
        }
        rc+=ple;
        if (fsize) {
          if (rc<=program_size) strcat(program, pls); else ferr=1;
        }
      }
    }
  }
  fseek(f,0, SEEK_SET);
  if (ferr) {
    rc=0;
    bc=0;
  }
  *param_size=bc;
  return rc;
}
//-------------------------------------------------------------------

int script_load(const char *fn)
{
  int sz;
  int  rcnt, bcnt, param_buf_size;
  FILE *fd = NULL;
  char *buf, *param_buf;

  param_buf=0;

  fd = fopen(fn, "rb");
  if (fd) {
    // Dateigre in sz
		fseek(fd,0, SEEK_END);
    sz = ftell(fd);
    fseek(fd,0, SEEK_SET);

    //first run only for buffer size
    rcnt = script_strip(fd, 0, 0, 0, &param_buf_size);
    bcnt = param_buf_size;
    printf("\nget buffer size: PROG=%d byte, PARAM=%d byte\n", rcnt, bcnt);
    if (rcnt > 0) {
      buf = malloc(rcnt+1);
      if (bcnt > 0) param_buf = malloc(bcnt+1);
      if(!buf ||!((bcnt>0 && param_buf)||(bcnt==0 && param_buf==0))) {
        fclose(fd);
        return 1;
      }
      printf("\nstriping ...");
      if (script_strip(fd, buf, rcnt, param_buf, &param_buf_size)==rcnt && param_buf_size==bcnt){
        printf("\nstripped size: PROG=%d byte, PARAM=%d byte\n", rcnt, bcnt);
        fclose(fd);
        remove ("STRIP.TXT");
        remove ("PARAM.TXT");
        fd = fopen("STRIP.TXT", "wb");
        printf("\nsave STRIP.TXT (%d byte)\n", rcnt);
        if (fd) fwrite(buf, 1, rcnt, fd); 
        if (param_buf) {
          fclose(fd);
          fd = fopen("PARAM.TXT", "wb");
          printf("save PARAM.TXT (%d byte)\n", bcnt);
          if (fd) fwrite(param_buf, 1, bcnt, fd);
        }
        printf("\n----------------------------------------\n");
        printf("old size: %d byte, new size: %d byte (saved %d%%)\n", sz, rcnt, 100-((1000*rcnt/sz)+5)/10);
        printf("----------------------------------------\n");
      }  
    }
  } else return 1;
  fclose(fd);
  if (param_buf) free(param_buf);
  free(buf);
  return 0;
}

//-------------------------------------------------------------------
int main(int argc, char *argv[])
{
	int res=0;
    
  if (argc != 1) {
    assert(argc == 2);
    printf("\n========================================\n");
    printf("input file: %s\n", argv[1]);
    if (script_load(argv[1])) {
      perror("Error opening file\n");
      exit(-1);
		}
  }
  return 0;
}
/*---------------------------------------------------------------------------*/
